@@ -39,6 +39,7 @@ import os
 import sys
 import time
 import logging
+import collections
 import dns.resolver
 import dns.tsig
 import dns.tsigkeyring
@@ -243,7 +244,7 @@ def create_txt_record(
         logger.info(' + Creating TXT record "%s" for the domain %s'
                     % (token, dn))
 
-        for i in range(1, len(domain_list)):
+        for i in range(0, len(domain_list)):
             head = '.'.join(domain_list[:i])
             tail = '.'.join(domain_list[i:])
             update = dns.update.Update(
@@ -336,7 +337,7 @@ def delete_txt_record(
             ' + Deleting TXT record "%s" for the domain %s' % (token, dn)
             )
 
-        for i in range(1, len(domain_list)):
+        for i in range(0, len(domain_list)):
             head = '.'.join(domain_list[:i])
             tail = '.'.join(domain_list[i:])
             # Attempt to delete the TXT record
@@ -628,10 +629,11 @@ def parse_args():
         help="Lower verbosity",
         action='count', default=0)
 
-    subparsers = parser.add_subparsers(help='sub-command help')
+    subparsers = parser.add_subparsers(help='sub-command help', dest='_hook')
 
     parser_deploychallenge = subparsers.add_parser(
         'deploy_challenge',
+        prefix_chars='+',
         help='make ACME challenge available via DNS')
     parser_deploychallenge.set_defaults(
         _func=deploy_challenge,
@@ -658,6 +660,7 @@ def parse_args():
 
     parser_cleanchallenge = subparsers.add_parser(
         'clean_challenge',
+        prefix_chars='+',
         help='remove ACME challenge from DNS')
     parser_cleanchallenge.set_defaults(
         _func=clean_challenge,
@@ -684,6 +687,7 @@ def parse_args():
 
     parser_deploycert = subparsers.add_parser(
         'deploy_cert',
+        prefix_chars='+',
         help='deploy certificate obtained from ACME [NO-OP]')
     parser_deploycert.set_defaults(
         _func=deploy_cert,
@@ -722,6 +726,7 @@ def parse_args():
 
     parser_unchangedcert = subparsers.add_parser(
         'unchanged_cert',
+        prefix_chars='+',
         help='unchanged certificate obtained from ACME [NO-OP]')
     parser_unchangedcert.set_defaults(
         _func=unchanged_cert,
@@ -756,6 +761,7 @@ def parse_args():
 
     parser_invalid_challenge = subparsers.add_parser(
         'invalid_challenge',
+        prefix_chars='+',
         help='challenge response has failed [NO-OP]')
     parser_invalid_challenge.set_defaults(
         _func=invalid_challenge,
@@ -778,6 +784,7 @@ def parse_args():
 
     parser_request_failure = subparsers.add_parser(
         'request_failure',
+        prefix_chars='+',
         help='challenge response has failed [NO-OP]')
     parser_request_failure.set_defaults(
         _func=request_failure,
@@ -804,6 +811,7 @@ def parse_args():
 
     parser_startup_hook = subparsers.add_parser(
         'startup_hook',
+        prefix_chars='+',
         help='dehydrated is starting up (do some initial tasks) [NO-OP]')
     parser_startup_hook.set_defaults(
         _func=startup_hook,
@@ -811,11 +819,20 @@ def parse_args():
 
     parser_exit_hook = subparsers.add_parser(
         'exit_hook',
+        prefix_chars='+',
         help='dehydrated is shutting down (do some final tasks) [NO-OP]')
     parser_exit_hook.set_defaults(
         _func=exit_hook,
         _parser=parser_exit_hook)
 
+    # prepare to ignore unrecognised hooks
+    fallback_parser = argparse.ArgumentParser()
+    fallback_parser.add_argument('_hook_arg', nargs='*')
+    subparsers._name_parser_map = collections.defaultdict(
+        lambda: fallback_parser,
+        subparsers._name_parser_map)
+    subparsers.choices = None
+
     args = parser.parse_args()
     try:
         while(args._extra[0]):
@@ -835,6 +852,12 @@ def parse_args():
     set_verbosity(verbosity)
 
     cfg = read_config(args)
+
+    # ignore unrecognised hooks
+    if not hasattr(args, '_func'):
+        logger.debug("ignoring unknown hook: %s", args._hook)
+        return (lambda cfg: None, cfg)
+
     return (args._func, cfg)
 
 
